# Generated via
#  `rails generate hyrax:work CrcDataset`
module Hyrax
  class CrcDatasetPresenter < Hyrax::WorkShowPresenter
    delegate :approval_number, :complex_date, :complex_funding_reference, :complex_identifier, :complex_person,
             :complex_relation, :complex_subject, :coverage, :crc_resource_type, :crc_work_type, :doi, :experiment_title,
             :experiment_description, :extra_information, :modality, :parent_work_id, :software_version, to: :solr_document

    def list_all_children(options = {})
      all_children = {
        'experiment' => {},
        'subject' => {},
        'session' => {},
        'modality' => {}
      }
      experiment = CrcDataset.find(self.id)
      members = experiment.file_sets.select{ |f| f.for_complex_identifier.nil? } + experiment.complex_subjects

      members = Kaminari.paginate_array(members).page(options[:page] || 1).per(options[:per_page] || 5)

      all_children['experiment'][id] = gather_children(members)
      # get subject children
      experiment.complex_subjects.each do |subject|
        subject_members = subject.file_sets + subject.complex_sessions
        all_children['subject'][subject.id] = gather_children(subject_members)
        # get session children
        subject.complex_sessions.each do |session|
          session_members = session.file_sets + session.complex_modalities
          all_children['session'][session.id] = gather_children(session_members)
          # get modality children
          session.complex_modalities.each do |modality|
            modality_members = modality.file_sets
            all_children['modality'][modality.id] = gather_children(modality_members)
          end
        end
      end
      all_children
    end

    private

    def gather_children(members)
      data = {'files' => [], 'children' => [], 'info' => {}}

      return {} unless members.any?

      members.each do |member|
        if member.class.name == "FileSet"
          # When the data has not been processed by the actors, 
          # the label property is missing and it throws http errors
          data['info'][member.id] = {'title' => member.title.first, 'label' => member&.label}
          data['files'] << member.id
        else
          if member.is_a?(ComplexSubject)
            data['info'][member.id] = {'title' => member.subject_identifier}
          elsif member.is_a?(ComplexSession)
            data['info'][member.id] = {'title' => member.session_title}
          elsif member.is_a?(ComplexModality)
            data['info'][member.id] = {'title' => member.modality}
          end

          data['children'] << member.id
        end
      end

      data
    end

  end
end
