# frozen_string_literal: true
module Hyrax
  module Workflow
    ##
    # This is a built in function for workflow, so that a workflow action can be created that
    # grants the creator the ability to alter it.
    module GrantEditToCrcManager
      # @param [#read_users=, #read_users] target (likely an ActiveRecord::Base) to which we are adding edit_users for the depositor
      # @return void
      def self.call(target:, **)

        crc_dataset = ::CrcDataset.find(target.id.id)
        collection = crc_dataset.parent_collections.first

        group_key = "#{collection.collection_type.title.parameterize(separator: '_')}_manager"

        crc_dataset.update(edit_groups: crc_dataset.edit_groups.to_a + [group_key])

        GrantEditToGroupJob.perform_later(crc_dataset, group_key)
      end
    end
  end
end