require 'csv'
namespace :rdms do
  desc 'Create Relationships, will read from specified file usage: rdms:create_relationships["tmp/imports/122_20221017111421/folder_data.csv"]'
  task :"create_relationships", [:file_path] => :environment do |task, args|
    parsed_metadata = CSV.parse(File.read(Rails.root.join(args.file_path)), headers: true)
    relationships = {
      collection_parent_work_child: {},
      work_parent_work_child: {}
    }
    records = {}
    parsed_metadata.each do |row|
      child_record = get_record(row[4], records)

      row[5].split(';').each do |parent_identifier|
        parent_record = get_record(parent_identifier, records)
        if parent_record.is_a?(::Collection) and child_record.is_a?(::Collection)
          create_collection_relationship(parent_record, child_record)
        else
          gather_relationships(parent_record, child_record, relationships)
        end
      end
    end
    create_work_relationships(relationships, records)
  end

  def create_collection_relationship(parent_record, child_record)
    begin
      collection_parent_collection_child(parent_record, child_record) if child_record.is_a?(::Collection)
    rescue => e
      puts "Error creating relationship: #{$!}"
      # puts "Backtrace:\n\t#{e.backtrace.join("\n\t")}"
    end
  end

  def gather_relationships(parent_record, child_record, relationships)
    if child_record.is_a?(::CrcDataset)
      if parent_record.is_a?(::Collection) and child_record.member_of_collection_ids.exclude?(parent_record.id)
        relationships[:collection_parent_work_child][parent_record.id] ||= []
        relationships[:collection_parent_work_child][parent_record.id] << child_record.id
      elsif parent_record.is_a?(::CrcDataset) and parent_record.member_ids.exclude?(child_record)
        relationships[:work_parent_work_child][parent_record.id] ||= []
        relationships[:work_parent_work_child][parent_record.id] << child_record.id
      end
    end
  end

  def create_work_relationships(relationships, records)
    # collection parent work children
    relationships[:collection_parent_work_child].each do |parent_id, child_ids|
      parent_record = get_record(parent_id, records)
      begin
        collection_parent_work_children(parent_record, child_ids)
      rescue => e
        puts "Error creating relationship: #{$!}"
        # puts "Backtrace:\n\t#{e.backtrace.join("\n\t")}"
      end
    end
    # work parent work children
    relationships[:work_parent_work_child].each do |parent_id, child_ids|
      parent_record = get_record(parent_id, records)
      begin
        work_parent_work_children(parent_record, child_ids)
      rescue => e
        puts "Error creating relationship: #{$!}"
        # puts "Backtrace:\n\t#{e.backtrace.join("\n\t")}"
      end
    end
  end

  def collection_parent_work_children(parent_record, child_work_ids)
    parent_record.reindex_extent = Hyrax::Adapters::NestingIndexAdapter::LIMITED_REINDEX

    parent_record.add_member_objects(child_work_ids)
  end

  def work_parent_work_children(parent_record, child_record_ids)
    records_hash = {}
    child_record_ids.each_with_index do |child_record_id, index|
      records_hash[index] = {id: child_record_id}
    end
    user = User.find_by(email: 'admin@hyrax')
    attrs = { work_members_attributes: records_hash }
    parent_record.try(:reindex_extent=, Hyrax::Adapters::NestingIndexAdapter::LIMITED_REINDEX)
    env = Hyrax::Actors::Environment.new(parent_record, Ability.new(user), attrs)

    Hyrax::CurationConcern.actor.update(env)
  end

  def get_record(identifier, records)
    if records.include?(identifier)
      child_record = records[identifier]
    else
      child_record = ActiveFedora::Base.where(source_tesim: identifier).first
      records[identifier] = child_record
    end
    child_record
  end
end
